@@ -179,21 +179,13 @@ fn default_daemon_dir() -> PathBuf {
 }
 
 fn default_config_files() -> Vec<OsString> {
-    #[cfg(not(feature = "ignore_default_config_files"))]
-    {
-        let mut files = vec![OsString::from("electrs.toml")]; // cwd
-        if let Some(mut path) = home_dir() {
-            path.extend(&[".electrs", "config.toml"]);
-            files.push(OsString::from(path)) // home directory
-        }
-        files.push(OsString::from("/etc/electrs/config.toml")); // system-wide
-        files
-    }
-
-    #[cfg(feature = "ignore_default_config_files")]
-    {
-        vec![]
+    let mut files = vec![OsString::from("electrs.toml")]; // cwd
+    if let Some(mut path) = home_dir() {
+        path.extend(&[".electrs", "config.toml"]);
+        files.push(OsString::from(path)) // home directory
     }
+    files.push(OsString::from("/etc/electrs/config.toml")); // system-wide
+    files
 }
 
 impl Config {
