@@ -11,7 +11,11 @@ use bitcoincore_rpc::json;
 use rayon::prelude::*;
 use serde::ser::{Serialize, SerializeSeq, Serializer};
 
-use crate::{daemon::Daemon, types::ScriptHash};
+use crate::{
+    daemon::Daemon,
+    metrics::{Gauge, Metrics},
+    types::ScriptHash,
+};
 
 pub(crate) struct Entry {
     pub txid: Txid,
@@ -26,7 +30,10 @@ pub(crate) struct Mempool {
     entries: HashMap<Txid, Entry>,
     by_funding: BTreeSet<(ScriptHash, Txid)>,
     by_spending: BTreeSet<(OutPoint, Txid)>,
-    histogram: Histogram,
+    fees: FeeHistogram,
+    // stats
+    vsize: Gauge,
+    count: Gauge,
 }
 
 // Smallest possible txid
@@ -40,17 +47,27 @@ fn txid_max() -> Txid {
 }
 
 impl Mempool {
-    pub fn new() -> Self {
+    pub fn new(metrics: &Metrics) -> Self {
         Self {
             entries: Default::default(),
             by_funding: Default::default(),
             by_spending: Default::default(),
-            histogram: Histogram::empty(),
+            fees: FeeHistogram::empty(),
+            vsize: metrics.gauge(
+                "mempool_txs_vsize",
+                "Total vsize of mempool transactions (in bytes)",
+                "fee_rate",
+            ),
+            count: metrics.gauge(
+                "mempool_txs_count",
+                "Total number of mempool transactions",
+                "fee_rate",
+            ),
         }
     }
 
-    pub(crate) fn fees_histogram(&self) -> &Histogram {
-        &self.histogram
+    pub(crate) fn fees_histogram(&self) -> &FeeHistogram {
+        &self.fees
     }
 
     pub(crate) fn get(&self, txid: &Txid) -> Option<&Entry> {
@@ -115,7 +132,14 @@ impl Mempool {
         for (txid, tx, entry) in entries {
             self.add_entry(*txid, tx, entry);
         }
-        self.histogram = Histogram::new(self.entries.values().map(|e| (e.fee, e.vsize)));
+        self.fees = FeeHistogram::new(self.entries.values().map(|e| (e.fee, e.vsize)));
+        for i in 1..FeeHistogram::BINS {
+            let bin_index = FeeHistogram::BINS - i - 1; // from 63 to 0
+            let limit = 1u64 << i;
+            let label = format!("[{:20.0}, {:20.0})", limit / 2, limit);
+            self.vsize.set(&label, self.fees.vsize[bin_index] as f64);
+            self.count.set(&label, self.fees.count[bin_index] as f64);
+        }
         debug!(
             "{} mempool txs: {} added, {} removed",
             self.entries.len(),
@@ -157,8 +181,9 @@ impl Mempool {
     }
 }
 
-pub(crate) struct Histogram {
-    /// bins[64-i] contains the total vsize of transactions with fee rate [2**(i-1), 2**i).
+pub(crate) struct FeeHistogram {
+    /// bins[64-i] contains transactions' statistics inside the fee band of [2**(i-1), 2**i).
+    /// bins[64] = [0, 1)
     /// bins[63] = [1, 2)
     /// bins[62] = [2, 4)
     /// bins[61] = [4, 8)
@@ -166,40 +191,54 @@ pub(crate) struct Histogram {
     /// ...
     /// bins[1] = [2**62, 2**63)
     /// bins[0] = [2**63, 2**64)
-    bins: [u64; Histogram::SIZE],
+    vsize: [u64; FeeHistogram::BINS],
+    count: [u64; FeeHistogram::BINS],
 }
 
-impl Histogram {
-    const SIZE: usize = 64;
+impl Default for FeeHistogram {
+    fn default() -> Self {
+        Self {
+            vsize: [0; FeeHistogram::BINS],
+            count: [0; FeeHistogram::BINS],
+        }
+    }
+}
+
+impl FeeHistogram {
+    const BINS: usize = 65; // 0..=64
 
     fn empty() -> Self {
         Self::new(std::iter::empty())
     }
 
     fn new(items: impl Iterator<Item = (Amount, u64)>) -> Self {
-        let mut bins = [0; Self::SIZE];
+        let mut result = FeeHistogram::default();
         for (fee, vsize) in items {
             let fee_rate = fee.as_sat() / vsize;
             let index = usize::try_from(fee_rate.leading_zeros()).unwrap();
             // skip transactions with too low fee rate (<1 sat/vB)
-            if let Some(bin) = bins.get_mut(index) {
+            if let Some(bin) = result.vsize.get_mut(index) {
                 *bin += vsize
             }
+            if let Some(bin) = result.count.get_mut(index) {
+                *bin += 1
+            }
         }
-        Self { bins }
+        result
     }
 }
 
-impl Serialize for Histogram {
+impl Serialize for FeeHistogram {
     fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
     where
         S: Serializer,
     {
-        let mut seq = serializer.serialize_seq(Some(self.bins.len()))?;
+        let mut seq = serializer.serialize_seq(Some(self.vsize.len()))?;
         // https://electrumx-spesmilo.readthedocs.io/en/latest/protocol-methods.html#mempool-get-fee-histogram
-        let fee_rates = (0..Histogram::SIZE).map(|i| std::u64::MAX >> i);
+        let fee_rates =
+            (0..FeeHistogram::BINS).map(|i| std::u64::MAX.checked_shr(i as u32).unwrap_or(0));
         fee_rates
-            .zip(self.bins.iter().copied())
+            .zip(self.vsize.iter().copied())
             .skip_while(|(_fee_rate, vsize)| *vsize == 0)
             .try_for_each(|element| seq.serialize_element(&element))?;
         seq.end()
@@ -208,7 +247,7 @@ impl Serialize for Histogram {
 
 #[cfg(test)]
 mod tests {
-    use super::Histogram;
+    use super::FeeHistogram;
     use bitcoin::Amount;
     use serde_json::json;
 
@@ -223,8 +262,12 @@ mod tests {
             (Amount::from_sat(50), 10),
             (Amount::from_sat(40), 10),
             (Amount::from_sat(80), 10),
+            (Amount::from_sat(1), 100),
         ];
-        let hist = json!(Histogram::new(items.into_iter()));
-        assert_eq!(hist, json!([[15, 10], [7, 40], [3, 20], [1, 10]]));
+        let hist = FeeHistogram::new(items.into_iter());
+        assert_eq!(
+            json!(hist),
+            json!([[15, 10], [7, 40], [3, 20], [1, 10], [0, 100]])
+        );
     }
 }
